@@ -1,8 +1,7 @@
 /**********************************************************************
- * $Id: Geometry.h 3538 2011-12-09 10:44:54Z strk $
  *
  * GEOS - Geometry Engine Open Source
- * http://geos.refractions.net
+ * http://geos.osgeo.org
  *
  * Copyright (C) 2009 2011 Sandro Santilli <strk@keybit.net>
  * Copyright (C) 2005 2006 Refractions Research Inc.
@@ -850,8 +849,6 @@ private:
 	///
 	const GeometryFactory *factory;
 
-	static const GeometryFactory* INTERNAL_GEOMETRY_FACTORY;
-
 	void* userData;
 };
 
@@ -893,67 +890,3 @@ struct GeomPtrPair {
 #endif
 
 #endif // ndef GEOS_GEOM_GEOMETRY_H
-
-/**********************************************************************
- * $Log$
- * Revision 1.14  2006/07/08 00:33:55  strk
- *         * configure.in: incremented CAPI minor version, to avoid                        falling behind any future version from the 2.2. branch.
- *         * source/geom/Geometry.cpp, source/geom/GeometryFactory.cpp,
- *         source/geomgraph/EdgeRing.cpp,
- *         source/headers/geos/geom/Geometry.h,
- *         source/headers/geos/geom/GeometryFactory.h,
- *         source/headers/geos/geom/GeometryFactory.inl,
- *         source/headers/geos/geomgraph/EdgeRing.h:
- *         updated doxygen comments (sync with JTS head).
- *         * source/headers/geos/platform.h.in: include <inttypes.h>
- *         rather then <stdint.h>
- *
- * Revision 1.13  2006/06/12 10:10:39  strk
- * Fixed getGeometryN() to take size_t rather then int, changed unsigned int parameters to size_t.
- *
- * Revision 1.12  2006/05/18 08:56:50  strk
- *         * source/geom/Geometry.cpp,
- *         source/headers/geos/geom/Geometry.h: added
- *         covers() and isCoveredBy() predicates.
- *         * tests/unit/Makefile.am,
- *         tests/unit/geom/Geometry/coversTest.cpp:
- *         added test for covers() predicates.
- *
- * Revision 1.11  2006/05/04 15:49:39  strk
- * updated all Geometry::getDimension() methods to return Dimension::DimensionType (closes bug#93)
- *
- * Revision 1.10  2006/04/28 10:55:39  strk
- * Geometry constructors made protected, to ensure all constructions use GeometryFactory,
- * which has been made friend of all Geometry derivates. getNumPoints() changed to return
- * size_t.
- *
- * Revision 1.9  2006/04/11 09:31:47  strk
- * Added Geometry::AutoPtr typedef
- *
- * Revision 1.8  2006/04/10 18:15:09  strk
- * Changed Geometry::envelope member to be of type auto_ptr<Envelope>.
- * Changed computeEnvelopeInternal() signater to return auto_ptr<Envelope>
- *
- * Revision 1.7  2006/04/07 09:54:30  strk
- * Geometry::getNumGeometries() changed to return 'unsigned int'
- * rather then 'int'
- *
- * Revision 1.6  2006/03/31 16:53:53  strk
- * Added comment about possible NULL return from getCoordinate()
- *
- * Revision 1.5  2006/03/24 09:52:41  strk
- * USE_INLINE => GEOS_INLINE
- *
- * Revision 1.4  2006/03/23 15:10:29  strk
- * Dropped by-pointer TopologyException constructor, various small cleanups
- *
- * Revision 1.3  2006/03/23 12:12:01  strk
- * Fixes to allow build with -DGEOS_INLINE
- *
- * Revision 1.2  2006/03/20 12:03:25  strk
- * Added operator<< for Geometry, writing HEXWKB
- *
- * Revision 1.1  2006/03/09 16:46:49  strk
- * geos::geom namespace definition, first pass at headers split
- *
- **********************************************************************/
