@@ -1,8 +1,7 @@
 /**********************************************************************
- * $Id: Geometry.cpp 3317 2011-04-28 15:08:47Z strk $
  *
  * GEOS - Geometry Engine Open Source
- * http://geos.refractions.net
+ * http://geos.osgeo.org
  *
  * Copyright (C) 2009 2011 Sandro Santilli <strk@keybit.net>
  * Copyright (C) 2005-2006 Refractions Research Inc.
@@ -35,9 +34,7 @@
 #include <geos/geom/MultiPolygon.h>
 #include <geos/geom/IntersectionMatrix.h>
 #include <geos/util/IllegalArgumentException.h>
-#include <geos/algorithm/CentroidPoint.h>
-#include <geos/algorithm/CentroidLine.h>
-#include <geos/algorithm/CentroidArea.h>
+#include <geos/algorithm/Centroid.h>
 #include <geos/algorithm/InteriorPointPoint.h>
 #include <geos/algorithm/InteriorPointLine.h>
 #include <geos/algorithm/InteriorPointArea.h>
@@ -63,6 +60,12 @@
 #include <cassert>
 #include <memory>
 
+#ifdef _MSC_VER
+#  ifdef MSVC_USE_VLD
+#    include <vld.h>
+#  endif
+#endif
+
 #define SHORTCIRCUIT_PREDICATES 1
 
 using namespace std;
@@ -100,9 +103,6 @@ jtsport()
 
 Geometry::GeometryChangedFilter Geometry::geometryChangedFilter;
 
-// REMOVE THIS, use GeometryFactory::getDefaultInstance() directly
-const GeometryFactory* Geometry::INTERNAL_GEOMETRY_FACTORY=GeometryFactory::getDefaultInstance();
-
 Geometry::Geometry(const GeometryFactory *newFactory)
 	:
 	envelope(NULL),
@@ -110,7 +110,7 @@ Geometry::Geometry(const GeometryFactory *newFactory)
 	userData(NULL)
 {
 	if ( factory == NULL ) {
-		factory = INTERNAL_GEOMETRY_FACTORY;
+		factory = GeometryFactory::getDefaultInstance();
 	} 
 	SRID=factory->getSRID();
 }
@@ -206,30 +206,8 @@ bool
 Geometry::getCentroid(Coordinate& ret) const
 {
 	if ( isEmpty() ) { return false; }
-
-	Coordinate c;
-
-	int dim=getDimension();
-	if(dim==0) {
-		CentroidPoint cent; 
-		cent.add(this);
-		if ( ! cent.getCentroid(c) )
-				return false;
-	} else if (dim==1) {
-		CentroidLine cent;
-		cent.add(this);
-		if ( ! cent.getCentroid(c) ) 
-			return false;
-	} else {
-		CentroidArea cent;
-		cent.add(this);
-		if ( ! cent.getCentroid(c) )
-			return false;
-	}
-
-	getPrecisionModel()->makePrecise(c);
-	ret=c;
-
+	if ( ! Centroid::getCentroid(*this, ret) ) return false;
+	getPrecisionModel()->makePrecise(ret); // not in JTS
 	return true;
 }
 
@@ -607,6 +585,39 @@ Geometry::symDifference(const Geometry *other) const
 	if ( isEmpty() ) return other->clone();
 	if ( other->isEmpty() ) return clone();
 
+	// if envelopes are disjoint return a MULTI geom or
+	// a geometrycollection
+	if ( ! getEnvelopeInternal()->intersects(other->getEnvelopeInternal()) )
+	{
+		const GeometryCollection *coll;
+
+		size_t ngeomsThis = getNumGeometries();
+		size_t ngeomsOther = other->getNumGeometries();
+
+		// Allocated for ownership transfer
+		vector<Geometry *> *v = new vector<Geometry *>();
+		v->reserve(ngeomsThis+ngeomsOther);
+
+
+		if ( NULL != (coll = dynamic_cast<const GeometryCollection *>(this)) )
+		{
+			for (size_t i=0; i<ngeomsThis; ++i)
+				v->push_back(coll->getGeometryN(i)->clone());
+		} else {
+			v->push_back(this->clone());
+		}
+
+		if ( NULL != (coll = dynamic_cast<const GeometryCollection *>(other)) )
+		{
+			for (size_t i=0; i<ngeomsOther; ++i)
+				v->push_back(coll->getGeometryN(i)->clone());
+		} else {
+			v->push_back(other->clone());
+		}
+
+		return factory->buildGeometry(v);
+	}
+
 	return BinaryOp(this, other, overlayOp(OverlayOp::opSYMDIFFERENCE)).release();
 }
 
