@@ -1,8 +1,7 @@
 /**********************************************************************
- * $Id: OverlayOp.cpp 3314 2011-04-28 14:16:41Z strk $
  *
  * GEOS - Geometry Engine Open Source
- * http://geos.refractions.net
+ * http://geos.osgeo.org
  *
  * Copyright (C) 2005-2006 Refractions Research Inc.
  * Copyright (C) 2001-2002 Vivid Solutions Inc.
@@ -14,9 +13,7 @@
  *
  ***********************************************************************
  *
- * Last port: operation/overlay/OverlayOp.java rev. 1.31 (JTS-1.10)
- *
- * NOTE: Use of EdgeNodingValidator is not strictly the same
+ * Last port: operation/overlay/OverlayOp.java r567 (JTS-1.12+)
  *
  **********************************************************************/
 
@@ -44,6 +41,7 @@
 #include <geos/geomgraph/DirectedEdge.h>
 #include <geos/geomgraph/Position.h>
 #include <geos/geomgraph/index/SegmentIntersector.h>
+#include <geos/util/Interrupt.h>
 #include <geos/util/TopologyException.h>
 #include <geos/geomgraph/EdgeNodingValidator.h>
 
@@ -97,10 +95,10 @@ OverlayOp::overlayOp(const Geometry *geom0, const Geometry *geom1,
 
 /* static public */
 bool
-OverlayOp::isResultOfOp(Label *label, OverlayOp::OpCode opCode)
+OverlayOp::isResultOfOp(const Label& label, OverlayOp::OpCode opCode)
 {
-	int loc0=label->getLocation(0);
-	int loc1=label->getLocation(1);
+	int loc0 = label.getLocation(0);
+	int loc1 = label.getLocation(1);
 	return isResultOfOp(loc0, loc1, opCode);
 }
 
@@ -246,7 +244,7 @@ OverlayOp::copyPoints(int argIndex)
 		Node* newNode=graph.addNode(graphNode->getCoordinate());
 		assert(newNode);
 
-		newNode->setLabel(argIndex,graphNode->getLabel()->getLocation(argIndex));
+		newNode->setLabel(argIndex, graphNode->getLabel().getLocation(argIndex));
 	}
 }
 
@@ -336,7 +334,7 @@ OverlayOp::updateNodeLabelling()
 		assert( dynamic_cast<DirectedEdgeStar*>(ees) );
 		DirectedEdgeStar* des = static_cast<DirectedEdgeStar*>(ees);
 		Label &lbl = des->getLabel();
-		node->getLabel()->merge(lbl);
+		node->getLabel().merge(lbl);
 #if GEOS_DEBUG
 		cerr<<"     "<<node->print()<<endl;
 #endif
@@ -357,10 +355,10 @@ OverlayOp::labelIncompleteNodes()
 			it != itEnd; ++it )
 	{
 		Node *n=it->second;
-		Label *label=n->getLabel();
+		const Label& label = n->getLabel();
 		if (n->isIsolated())
 		{
-			if (label->isNull(0))
+			if (label.isNull(0))
 			{
 				labelIncompleteNode(n,0);
 			}
@@ -389,7 +387,7 @@ OverlayOp::labelIncompleteNode(Node *n, int targetIndex)
 #endif
 	const Geometry *targetGeom = arg[targetIndex]->getGeometry();
 	int loc=ptLocator.locate(n->getCoordinate(), targetGeom);
-	n->getLabel()->setLocation(targetIndex,loc);
+	n->getLabel().setLocation(targetIndex, loc);
 
 #if GEOS_DEBUG
 	cerr<<"   after location set: "<<n->print()<<endl;
@@ -524,11 +522,11 @@ OverlayOp::findResultAreaEdges(OverlayOp::OpCode opCode)
 	{
 		DirectedEdge *de=(DirectedEdge*) (*ee)[i];
 		// mark all dirEdges with the appropriate label
-		Label *label=de->getLabel();
-		if (label->isArea()
+		const Label& label = de->getLabel();
+		if ( label.isArea()
 			&& !de->isInteriorAreaEdge()
-			&& isResultOfOp(label->getLocation(0,Position::RIGHT),
-					label->getLocation(1,Position::RIGHT),
+			&& isResultOfOp(label.getLocation(0,Position::RIGHT),
+					label.getLocation(1,Position::RIGHT),
 					opCode)
 			)
 		{
@@ -656,6 +654,8 @@ OverlayOp::computeOverlay(OverlayOp::OpCode opCode)
 	copyPoints(0);
 	copyPoints(1);
 
+	GEOS_CHECK_FOR_INTERRUPTS();
+
 	// node the input Geometries
 	delete arg[0]->computeSelfNodes(li,false);
 	delete arg[1]->computeSelfNodes(li,false);
@@ -664,6 +664,8 @@ OverlayOp::computeOverlay(OverlayOp::OpCode opCode)
 	cerr<<"OverlayOp::computeOverlay: computed SelfNodes"<<endl;
 #endif
 
+	GEOS_CHECK_FOR_INTERRUPTS();
+
 	// compute intersections between edges of the two input geometries
 	delete arg[0]->computeEdgeIntersections(arg[1], &li,true);
 
@@ -673,16 +675,22 @@ OverlayOp::computeOverlay(OverlayOp::OpCode opCode)
 #endif
 
 
+	GEOS_CHECK_FOR_INTERRUPTS();
+
 	vector<Edge*> baseSplitEdges;
 	arg[0]->computeSplitEdges(&baseSplitEdges);
 	arg[1]->computeSplitEdges(&baseSplitEdges);
 
+	GEOS_CHECK_FOR_INTERRUPTS();
+
 	// add the noded edges to this result graph
 	insertUniqueEdges(&baseSplitEdges);
 	computeLabelsFromDepths();
 	replaceCollapsedEdges();
 	//Debug.println(edgeList);
 
+	GEOS_CHECK_FOR_INTERRUPTS();
+
 #ifdef ENABLE_EDGE_NODING_VALIDATOR // {
 	/**
 	 * Check that the noding completed correctly.
@@ -695,44 +703,39 @@ OverlayOp::computeOverlay(OverlayOp::OpCode opCode)
 	 * In the future hopefully a faster check can be developed.
 	 *
 	 */
-	if ( resultPrecisionModel->isFloating() ) // NOTE: this is not in JTS
-	{
-
-		try
-		{
-			// Will throw TopologyException if noding is
-			// found to be invalid
-			EdgeNodingValidator::checkValid(edgeList.getEdges());
-		}
-		catch (const util::TopologyException& ex)
-		{
-#ifdef GEOS_DEBUG_VALIDATION // {
-			cout << "EdgeNodingValidator found noding invalid: " << ex.what() << endl;
-#endif // }
-                        // In the error scenario, the edgeList is not properly
-                        // deleted. Cannot add to the destructor of EdgeList
-                        // (as it should) because 
-                        // "graph.addEdges(edgeList.getEdges());" below
-                        // takes over edgeList ownership in the success case.
-                        edgeList.clearList();
-
-			throw ex;
-		}
-#ifdef GEOS_DEBUG_VALIDATION // {
+  try
+  {
+    // Will throw TopologyException if noding is
+    // found to be invalid
+    EdgeNodingValidator::checkValid(edgeList.getEdges());
+#ifdef GEOS_DEBUG_VALIDATION 
 		cout << "EdgeNodingValidator accepted the noding" << endl;
-#endif // }
+#endif
+  }
+  catch (const util::TopologyException& ex)
+  {
+#ifdef GEOS_DEBUG_VALIDATION
+    cout << "EdgeNodingValidator found noding invalid: " << ex.what() << endl;
+#endif
+
+    // In the error scenario, the edgeList is not properly
+    // deleted. Cannot add to the destructor of EdgeList
+    // (as it should) because 
+    // "graph.addEdges(edgeList.getEdges());" below
+    // takes over edgeList ownership in the success case.
+    edgeList.clearList();
+
+    throw ex;
+  }
 
-	}
-#ifdef GEOS_DEBUG_VALIDATION // {
-	else
-	{
-		cout << "Did not run EdgeNodingValidator as the precision model is not floating" << endl;
-	}
-#endif // GEOS_DEBUG_VALIDATION }
 #endif // ENABLE_EDGE_NODING_VALIDATOR }
 
+	GEOS_CHECK_FOR_INTERRUPTS();
+
 	graph.addEdges(edgeList.getEdges());
 
+	GEOS_CHECK_FOR_INTERRUPTS();
+
 	// this can throw TopologyException *
 	computeLabelling();
 
@@ -741,6 +744,7 @@ OverlayOp::computeOverlay(OverlayOp::OpCode opCode)
 	//Debug.printWatch();
 	//nodeMap.print(System.out);
 
+	GEOS_CHECK_FOR_INTERRUPTS();
 
 	/*
 	 * The ordering of building the result Geometries is important.
@@ -752,6 +756,8 @@ OverlayOp::computeOverlay(OverlayOp::OpCode opCode)
 	findResultAreaEdges(opCode);
 	cancelDuplicateResultEdges();
 
+	GEOS_CHECK_FOR_INTERRUPTS();
+
 	PolygonBuilder polyBuilder(geomFact);
 	
 	// might throw a TopologyException *
@@ -805,29 +811,30 @@ OverlayOp::insertUniqueEdge(Edge *e)
 #if GEOS_DEBUG
 		cerr<<"  found identical edge, should merge Z"<<endl;
 #endif
-		Label *existingLabel=existingEdge->getLabel();
 
-		Label *labelToMerge=e->getLabel();
+		Label& existingLabel = existingEdge->getLabel();
+
+		Label labelToMerge = e->getLabel();
 
 		// check if new edge is in reverse direction to existing edge
 		// if so, must flip the label before merging it
 		if (!existingEdge->isPointwiseEqual(e))
 		{
-//			labelToMerge=new Label(e->getLabel());
-			labelToMerge->flip();
+			labelToMerge.flip();
 		}
-		Depth &depth=existingEdge->getDepth();
+		Depth &depth = existingEdge->getDepth();
 
 		// if this is the first duplicate found for this edge,
 		// initialize the depths
 		if (depth.isNull())
 		{
-			depth.add(*existingLabel);
+			depth.add(existingLabel);
 		}
 
-		depth.add(*labelToMerge);
+		depth.add(labelToMerge);
+
+		existingLabel.merge(labelToMerge);
 
-		existingLabel->merge(*labelToMerge);
 		//Debug.print("inserted edge: "); Debug.println(e);
 		//Debug.print("existing edge: "); Debug.println(existingEdge);
 		dupEdges.push_back(e);
@@ -851,8 +858,8 @@ OverlayOp::computeLabelsFromDepths()
 	for(size_t j=0, s=edgeList.getEdges().size(); j<s; ++j)
 	{
 		Edge *e=edgeList.get(j);
-		Label *lbl=e->getLabel();
-		Depth &depth=e->getDepth();
+		Label& lbl = e->getLabel();
+		Depth &depth = e->getDepth();
 
 		/*
 		 * Only check edges for which there were duplicates,
@@ -864,7 +871,7 @@ OverlayOp::computeLabelsFromDepths()
 		depth.normalize();
 		for (int i=0;i<2;i++)
 		{
-			if (!lbl->isNull(i) && lbl->isArea() && !depth.isNull(i))
+			if (!lbl.isNull(i) && lbl.isArea() && !depth.isNull(i))
 			{
 				/*
 				 * if the depths are equal, this edge is the result of
@@ -873,7 +880,7 @@ OverlayOp::computeLabelsFromDepths()
 				 * so it has collapsed to a line.
 				 */
 				if (depth.getDelta(i)==0) {
-					lbl->toLine(i);
+					lbl.toLine(i);
 				} else {
 					/*
 					 * This edge may be the result of a dimensional collapse,
@@ -882,9 +889,9 @@ OverlayOp::computeLabelsFromDepths()
 					 * side locations indicated by the depth values.
 					 */
 					assert(!depth.isNull(i,Position::LEFT)); // depth of LEFT side has not been initialized
-					lbl->setLocation(i,Position::LEFT,depth.getLocation(i,Position::LEFT));
+					lbl.setLocation(i,Position::LEFT,depth.getLocation(i,Position::LEFT));
 					assert(!depth.isNull(i,Position::RIGHT)); // depth of RIGHT side has not been initialized
-					lbl->setLocation(i,Position::RIGHT,depth.getLocation(i,Position::RIGHT));
+					lbl.setLocation(i,Position::RIGHT,depth.getLocation(i,Position::RIGHT));
 				}
 			}
 		}
@@ -1023,29 +1030,3 @@ OverlayOp::checkObviouslyWrongResult(OverlayOp::OpCode opCode)
 } // namespace geos.operation
 } // namespace geos
 
-/**********************************************************************
- * $Log$
- * Revision 1.77  2006/07/05 20:19:29  strk
- * added checks for obviously wrong result of difference and intersection ops
- *
- * Revision 1.76  2006/06/14 15:38:16  strk
- * Fixed just-introduced bug
- *
- * Revision 1.75  2006/06/14 15:03:32  strk
- * * source/operation/overlay/OverlayOp.cpp: use NodeMap::container and related typedefs, removed (int) casts, optimized loops.
- *
- * Revision 1.74  2006/06/13 21:42:55  strk
- * trimmed cvs log, cleanups
- *
- * Revision 1.73  2006/06/12 11:29:24  strk
- * unsigned int => size_t
- *
- * Revision 1.72  2006/06/09 07:42:13  strk
- * * source/geomgraph/GeometryGraph.cpp, source/operation/buffer/OffsetCurveSetBuilder.cpp, source/operation/overlay/OverlayOp.cpp, source/operation/valid/RepeatedPointTester.cpp: Fixed warning after Polygon ring accessor methods changed to work with size_t. Small optimizations in loops.
- *
- * Revision 1.71  2006/06/05 15:36:34  strk
- * Given OverlayOp funx code enum a name and renamed values to have a lowercase prefix. Drop all of noding headers from installed header set.
- *
- *
- **********************************************************************/
-
