@@ -3,6 +3,8 @@
 require 'test/unit'
 require 'geos'
 
+# NOTE - These tests are no longer valid with the C-API
+
 class TestGeosEnvelope < Test::Unit::TestCase
   def check(envelope)
     assert_equal(5, envelope.get_min_x())
@@ -37,6 +39,6 @@ class TestGeosEnvelope < Test::Unit::TestCase
     envelope = Geos::Envelope.new(5, 10, 6, 11)
     string = envelope.to_s
 
-    assert_equal("<Envelope Env[5:10,6:11]>", string)
+    assert_equal("<Geos::Envelope Env[5:10,6:11]>", string)
   end
 end
\ No newline at end of file
