@@ -1,10 +1,8 @@
-// $Id: computeOrientationTest.cpp 2156 2008-08-12 00:23:44Z mloskot $
 // 
 // Test Suite for CGAlgorithms::computeOrientation() function
 // Ported from JTS junit/algorithm/ComputeOrientationTest.java
 
-// tut
-#include <tut.h>
+#include <tut.hpp>
 // geos
 #include <geos/algorithm/CGAlgorithms.h>
 #include <geos/geom/Coordinate.h>
