@@ -1,4 +1,3 @@
-// $Id: LineStringTest.cpp 2344 2009-04-09 21:46:30Z mloskot $
 // 
 // Test Suite for geos::geom::LineString class.
 
@@ -129,6 +128,7 @@ namespace tut
 		ensure_equals( ls->getGeometryTypeId(), geos::geom::GEOS_LINESTRING );
 		ensure_equals( ls->getDimension(), geos::geom::Dimension::L );
 		ensure_equals( ls->getBoundaryDimension(), geos::geom::Dimension::P );
+        ensure_equals( ls->getCoordinateDimension(), 3 );
 		ensure_equals( ls->getNumPoints(), size3 );
 		ensure_equals( ls->getArea(), 0.0 );
 		ensure( ls->getLength() != 0.0 );
@@ -328,6 +328,7 @@ namespace tut
 		ensure( !line->isEmpty() );
 		ensure( !line->isClosed() );
 		ensure( !line->isRing() );
+        ensure( line->getCoordinateDimension() == 2 );
 		
 		// FREE TESTED LINESTRING
 		factory_.destroyGeometry(line);
@@ -341,7 +342,7 @@ namespace tut
 		GeometryPtr geo = reader_.read("LINESTRING (0 0, 5 5, 10 5, 10 10)");
 		ensure( geo != 0 );
 
-		LineStringPtr line = static_cast<LineStringPtr>(geo);
+		LineStringPtr line = dynamic_cast<LineStringPtr>(geo);
 		ensure(line != 0);
 		
 		GeometryPtr envelope = line->getEnvelope();	
@@ -363,7 +364,7 @@ namespace tut
 		GeometryPtr geo = reader_.read("LINESTRING (0 0, 5 5, 10 5, 10 10)");
 		ensure( geo != 0 );
 
-		LineStringPtr line = static_cast<LineStringPtr>(geo);
+		LineStringPtr line = dynamic_cast<LineStringPtr>(geo);
 		ensure(line != 0);
 		
 		GeometryPtr boundary = line->getBoundary();	
@@ -385,7 +386,7 @@ namespace tut
 		GeometryPtr geo = reader_.read("LINESTRING (0 0, 5 5, 10 5, 10 10)");
 		ensure( geo != 0 );
 
-		LineStringPtr line = static_cast<LineStringPtr>(geo);
+		LineStringPtr line = dynamic_cast<LineStringPtr>(geo);
 		ensure(line != 0);
 		
 		GeometryPtr hull = line->convexHull();	
