@@ -1,4 +1,3 @@
-// $Id: PointTest.cpp 2344 2009-04-09 21:46:30Z mloskot $
 // 
 // Test Suite for geos::geom::Point class.
 
@@ -50,7 +49,7 @@ namespace tut
 	    // Create non-empty Point
 	    GeometryPtr geo = 0;
 	    geo = reader_.read("POINT(1.234 5.678)");
-	    point_ = static_cast<PointPtr>(geo);
+	    point_ = dynamic_cast<PointPtr>(geo);
 	}
 	
 	~test_point_data()
@@ -91,6 +90,10 @@ namespace tut
 
 		PointAutoPtr point(factory_.createPoint(coords));
 		ensure( !point->isEmpty() );
+        
+        // currently the empty CoordinateArraySequence constructor 
+        // produces a dimension 3 sequence.
+        ensure( point->getCoordinateDimension() == 3 );
     }
 
     // Test of user's constructor throwing IllegalArgumentException
@@ -510,5 +513,23 @@ namespace tut
 		factory_.destroyGeometry(p3);
 	}
 
+	// Test of getCoordinateDimension() for 2d/3d.
+	template<>
+	template<>
+	void object::test<40>()
+	{
+		GeometryPtr p = reader_.read("POINT(-1.233 5.678 1.0)");
+
+        ensure( p->getCoordinateDimension() == 3 );
+
+        delete p;
+
+		p = reader_.read("POINT(-1.233 5.678)");
+
+        ensure( p->getCoordinateDimension() == 2 );
+
+        delete p;
+    }
+
 } // namespace tut
 
