@@ -1,4 +1,3 @@
-// $Id: SegmentNodeTest.cpp 2344 2009-04-09 21:46:30Z mloskot $
 // 
 // Test Suite for geos::noding::SegmentNode class.
 
@@ -55,7 +54,7 @@ namespace tut
 
         // Create coordinates sequence
         const size_t coords_size = 2;
-        CoordSeqPtr cs( factory_->create(0, coords_size) );
+        CoordSeqPtr cs( factory_->create((size_t)0, coords_size) );
 
         ensure( 0 != cs.get() );
 
@@ -68,7 +67,7 @@ namespace tut
 
         // Create SegmentString instance
         
-        NodedSegmentString segment(cs.get(), 0);
+        NodedSegmentString segment(cs.release(), 0);
 
         ensure_equals( segment.size(), coords_size );
 
@@ -103,7 +102,7 @@ namespace tut
 
         // Create coordinates sequence
         const size_t coords_size = 2;
-        CoordSeqPtr cs( factory_->create(0, coords_size) );
+        CoordSeqPtr cs( factory_->create((size_t)0, coords_size) );
 
         ensure( 0 != cs.get() );
 
@@ -116,7 +115,7 @@ namespace tut
 
         // Create SegmentString instance
         
-        NodedSegmentString segment(cs.get(), 0);
+        NodedSegmentString segment(cs.release(), 0);
 
         ensure_equals( segment.size(), coords_size );
 
@@ -145,7 +144,7 @@ namespace tut
 
         // Create coordinates sequence
         const size_t coords_size = 2;
-        CoordSeqPtr cs( factory_->create(0, coords_size) );
+        CoordSeqPtr cs( factory_->create((size_t)0, coords_size) );
 
         ensure( 0 != cs.get() );
 
@@ -158,7 +157,7 @@ namespace tut
 
         // Create SegmentString instance
         
-        NodedSegmentString segment(cs.get(), 0);
+        NodedSegmentString segment(cs.release(), 0);
 
         ensure_equals( segment.size(), coords_size );
 
@@ -187,7 +186,7 @@ namespace tut
 
         // Create coordinates sequence
         const size_t coords_size = 2;
-        CoordSeqPtr cs( factory_->create(0, coords_size) );
+        CoordSeqPtr cs( factory_->create((size_t)0, coords_size) );
 
         ensure( 0 != cs.get() );
 
@@ -200,7 +199,7 @@ namespace tut
 
         // Create SegmentString instance
         
-        NodedSegmentString segment(cs.get(), 0);
+        NodedSegmentString segment(cs.release(), 0);
 
         ensure_equals( segment.size(), coords_size );
 
