@@ -1,10 +1,8 @@
-// $Id: IsSimpleOpTest.cpp 2156 2008-08-12 00:23:44Z mloskot $
 // 
 // Test Suite for geos::operation::IsSimpleOp class
 // Ported from JTS junit/operation/IsSimpleTest.java
 
-// tut
-#include <tut.h>
+#include <tut.hpp>
 // geos
 #include <geos/operation/IsSimpleOp.h>
 #include <geos/geom/Coordinate.h>
