@@ -1,4 +1,3 @@
-// $Id: CascadedPolygonUnionTest.cpp 2344 2009-04-09 21:46:30Z mloskot $
 // 
 // Test Suite for geos::operation::geounion::CascadedPolygonUnion class.
 
@@ -24,7 +23,7 @@ namespace tut
     //
 
     // Common data used by tests
-    struct test_unaryuniontest_data
+    struct test_cascadedpolygonuniontest_data
     {
         geos::geom::GeometryFactory gf;
         geos::io::WKTReader wktreader;
@@ -32,16 +31,16 @@ namespace tut
 
         typedef geos::geom::Geometry::AutoPtr GeomPtr;
 
-        test_unaryuniontest_data()
+        test_cascadedpolygonuniontest_data()
           : gf(),
             wktreader(&gf)
         {}
     };
 
-    typedef test_group<test_unaryuniontest_data> group;
+    typedef test_group<test_cascadedpolygonuniontest_data> group;
     typedef group::object object;
 
-    group test_unaryuniontest_group("geos::operation::geounion::CascadedPolygonUnion");
+    group test_cascadedpolygonuniontest_group("geos::operation::geounion::CascadedPolygonUnion");
 
     // test runner
     geos::geom::Geometry* unionIterated(
@@ -72,7 +71,7 @@ namespace tut
         return CascadedPolygonUnion::Union(geoms);
     }
 
-    void test_runner(test_unaryuniontest_data& t,
+    void test_runner(test_cascadedpolygonuniontest_data& t,
         std::vector<geos::geom::Polygon*>* geoms) 
     {
         std::auto_ptr<geos::geom::Geometry> union1(unionIterated(geoms));
@@ -114,7 +113,7 @@ namespace tut
         {
             std::string wkt(*p);
             geos::geom::Polygon* geom = 
-                static_cast<geos::geom::Polygon*>(wktreader.read(wkt));
+                dynamic_cast<geos::geom::Polygon*>(wktreader.read(wkt));
             g.push_back(geom);
         }
 
@@ -130,7 +129,7 @@ namespace tut
             for (int j = 0; j < num; ++j) {
                 std::auto_ptr<geos::geom::Point> pt(
                     gf.createPoint(geos::geom::Coordinate(i, j)));
-                g->push_back(static_cast<geos::geom::Polygon*>(pt->buffer(radius)));
+                g->push_back(dynamic_cast<geos::geom::Polygon*>(pt->buffer(radius)));
             }
         }
     }
