@@ -1,8 +1,7 @@
 /**********************************************************************
- * $Id: XMLTester.cpp 2760 2009-12-03 14:48:08Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
- * http://geos.refractions.net
+ * http://geos.osgeo.org
  *
  * Copyright (C) 2005-2006 Refractions Research Inc.
  * Copyright (C) 2001-2002 Vivid Solutions Inc.
@@ -74,11 +73,8 @@
 #include "Stackwalker.h"
 #endif
 
-// Undefine this to use Geometry methods for overlay operations
-//
-// Using BinaryOp gives more successes though...
-//
-#define USE_BINARYOP 1
+// Geometry methods do use BinaryOp internally
+#undef USE_BINARYOP
 
 using namespace geos;
 using namespace geos::operation::polygonize;
@@ -384,7 +380,9 @@ XMLTester::run(const std::string &source)
 
 	if ( ! xml.LoadFile(source.c_str()) )
 	{
-		std::cerr << "Error loading " << source << std::endl;
+		std::stringstream err;
+		err << "Could not load " << source << ": " << xml.ErrorDesc();
+		throw runtime_error(err.str());
 	}
 
 	//dump_to_stdout(&xml);
@@ -416,6 +414,8 @@ XMLTester::resetCounters()
 void
 XMLTester::parseRun(const TiXmlNode* node)
 {
+	using geos::geom::PrecisionModel;
+
 	assert(node);
 
 	//dump_to_stdout(node);
@@ -423,6 +423,18 @@ XMLTester::parseRun(const TiXmlNode* node)
 	// Look for precisionModel element
 	const TiXmlElement* el = node->FirstChildElement("precisionModel");
 	if ( el ) parsePrecisionModel(el);
+	else pm.reset(new PrecisionModel());
+
+	if (verbose > 1)
+	{
+		std::cerr << *curr_file <<": run: Precision Model: " << pm->toString() <<std::endl;
+	}
+
+	factory.reset(new geom::GeometryFactory(pm.get()));
+	wktreader.reset(new io::WKTReader(factory.get()));
+	wktwriter.reset(new io::WKTWriter());
+	wkbreader.reset(new io::WKBReader(*factory));
+	wkbwriter.reset(new io::WKBWriter());
 
 	const TiXmlNode* casenode;
 	for ( casenode = node->FirstChild("case");
@@ -478,39 +490,25 @@ XMLTester::parsePrecisionModel(const TiXmlElement* el)
 		// NOTE: PrecisionModel discards offsets anyway...
 		pm.reset(new PrecisionModel(scale, offsetX, offsetY));
 	}
-
-	if (verbose > 1)
-	{
-		std::cerr << *curr_file <<": run: Precision Model: " << pm->toString() <<std::endl;
-	}
-
-	factory.reset(new geom::GeometryFactory(pm.get()));
-	wktreader.reset(new io::WKTReader(factory.get()));
-	wktwriter.reset(new io::WKTWriter());
-	wkbreader.reset(new io::WKBReader(*factory));
-	wkbwriter.reset(new io::WKBWriter());
 }
 
 
-void
+bool
 XMLTester::testValid(const geom::Geometry* g, const std::string& label)
 {
 	operation::valid::IsValidOp ivo(g);
-	bool result;
-	result = ivo.isValid();
-	if ( result == 0 )
+	bool valid = ivo.isValid();
+	if ( ! valid )
 	{
 		operation::valid::TopologyValidationError *err = ivo.getValidationError();
-		if ( err ) {
-			std::cerr << *curr_file << ":"
-			          << " case" << caseCount << ":"
-			          << " test" << testCount << ": "
-				  << opSignature << ": " 
-			          << " invalid geometry (" << label 
-			          << "): " << err->toString() << std::endl;
-		}
+		std::cerr << *curr_file << ":"
+			  << " case" << caseCount << ":"
+			  << " test" << testCount << ": "
+			  << opSignature << ": " 
+			  << " invalid geometry (" << label 
+			  << "): " << err->toString() << std::endl;
 	}
-	//return result;
+	return valid;
 }
 
 /**
@@ -650,13 +648,13 @@ XMLTester::parseCase(const TiXmlNode* node)
 
 /*private*/
 void
-XMLTester::printGeom(std::ostream& os, geom::Geometry *g)
+XMLTester::printGeom(std::ostream& os, const geom::Geometry *g)
 {
 	os << printGeom(g);
 }
 
 std::string
-XMLTester::printGeom(geom::Geometry *g)
+XMLTester::printGeom(const geom::Geometry *g)
 {
 	if ( HEXWKB_output )
 	{
@@ -797,32 +795,35 @@ XMLTester::parseTest(const TiXmlNode* node)
 
 			if (gRes->compareTo(gRealRes.get())==0) success=1;
 
-			if ( testValidOutput ) testValid(gRes.get(), "result");
-
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
 
+			if ( testValidOutput )
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 		else if (opName=="union")
 		{
 			GeomAutoPtr gRes(parseGeometry(opRes, "expected"));
-			gRes->normalize();
 
+			GeomAutoPtr gRealRes;
+			if ( gB ) {
 #ifndef USE_BINARYOP
-			GeomAutoPtr gRealRes(gA->Union(gB));
+				gRealRes.reset(gA->Union(gB));
 #else
-			GeomAutoPtr gRealRes = BinaryOp(gA, gB, overlayOp(OverlayOp::opUNION));
+				gRealRes = BinaryOp(gA, gB, overlayOp(OverlayOp::opUNION));
 #endif
+			} else {
+				gRealRes = gA->Union();
+			}
 
-			gRealRes->normalize();
-
-			if (gRes->compareTo(gRealRes.get())==0) success=1;
-
-			if ( testValidOutput ) testValid(gRes.get(), "result");
+			if (gRes->equals(gRealRes.get())) success=1;
 
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
+
+			if ( testValidOutput )
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 		else if (opName=="difference")
@@ -841,11 +842,11 @@ XMLTester::parseTest(const TiXmlNode* node)
 
 			if (gRes->compareTo(gRealRes.get())==0) success=1;
 
-			if ( testValidOutput ) testValid(gRes.get(), "result");
-
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
 
+			if ( testValidOutput )
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 		else if (opName=="symdifference")
@@ -863,11 +864,11 @@ XMLTester::parseTest(const TiXmlNode* node)
 
 			if (gRes->compareTo(gRealRes.get())==0) success=1;
 
-			if ( testValidOutput ) testValid(gRes.get(), "result");
-
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
 
+			if ( testValidOutput )
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 		else if (opName=="intersects")
@@ -938,11 +939,11 @@ XMLTester::parseTest(const TiXmlNode* node)
 
 			if (gRes->compareTo(gRealRes.get())==0) success=1;
 
-			if ( testValidOutput ) testValid(gRes.get(), "result");
-
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
 
+			if ( testValidOutput )
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 		else if (opName=="getcentroid")
@@ -956,16 +957,16 @@ XMLTester::parseTest(const TiXmlNode* node)
 			GeomAutoPtr gRealRes(gT->getCentroid());
 
 			if ( gRealRes.get() ) gRealRes->normalize();
-			else gRealRes.reset(factory->createGeometryCollection());
+			else gRealRes.reset(factory->createPoint());
 			gRealRes->normalize();
 
 			if (gRes->compareTo(gRealRes.get())==0) success=1;
 
-			if ( testValidOutput ) testValid(gRes.get(), "result");
-
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
 
+			if ( testValidOutput ) 
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 		else if (opName=="issimple")
@@ -993,11 +994,11 @@ XMLTester::parseTest(const TiXmlNode* node)
 
 			if (gRes->compareTo(gRealRes.get())==0) success=1;
 
-			if ( testValidOutput ) testValid(gRes.get(), "result");
-
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
 
+			if ( testValidOutput ) 
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 		else if (opName=="buffer")
@@ -1030,10 +1031,11 @@ XMLTester::parseTest(const TiXmlNode* node)
 			// Validate the buffer operation
 			success = checkBufferSuccess(*gRes, *gRealRes, dist);
 
-			if ( testValidOutput ) testValid(gRes.get(), "result");
-
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
+
+			if ( testValidOutput )
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 		else if (opName=="buffersinglesided")
@@ -1074,10 +1076,11 @@ XMLTester::parseTest(const TiXmlNode* node)
 			success = checkSingleSidedBufferSuccess(*gRes,
 					*gRealRes, dist);
 
-			if ( testValidOutput ) testValid(gRes.get(), "result");
-
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
+
+			if ( testValidOutput )
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 		else if (opName=="buffermitredjoin")
@@ -1111,10 +1114,11 @@ XMLTester::parseTest(const TiXmlNode* node)
 			// Validate the buffer operation
 			success = checkBufferSuccess(*gRes, *gRealRes, dist);
 
-			if ( testValidOutput ) testValid(gRes.get(), "result");
-
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
+
+			if ( testValidOutput ) 
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 
@@ -1128,15 +1132,15 @@ XMLTester::parseTest(const TiXmlNode* node)
 
 			GeomAutoPtr gRealRes(gT->getInteriorPoint());
 			if ( gRealRes.get() ) gRealRes->normalize();
-			else gRealRes.reset(factory->createGeometryCollection());
+			else gRealRes.reset(factory->createPoint());
 
 			if (gRes->compareTo(gRealRes.get())==0) success=1;
 
-			if ( testValidOutput ) testValid(gRes.get(), "result");
-
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
 
+			if ( testValidOutput ) 
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 		else if (opName=="iswithindistance")
@@ -1174,11 +1178,11 @@ XMLTester::parseTest(const TiXmlNode* node)
 
 			if (gRes->compareTo(gRealRes.get())==0) success=1;
 
-			if ( testValidOutput ) testValid(gRes.get(), "result");
-
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
 
+			if ( testValidOutput )
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 		else if (opName=="linemerge")
@@ -1201,17 +1205,19 @@ XMLTester::parseTest(const TiXmlNode* node)
 
 			if (gRes->compareTo(gRealRes.get())==0) success=1;
 
-			if ( testValidOutput ) testValid(gRes.get(), "result");
-
 			actual_result=printGeom(gRealRes.get());
 			expected_result=printGeom(gRes.get());
 
+			if ( testValidOutput ) 
+				success &= testValid(gRealRes.get(), "result");
 		}
 
 		else if (opName=="areatest")
 		{
 			char* rest;
 			double toleratedDiff = std::strtod(opRes.c_str(), &rest);
+			int validOut = 1;
+
 			if ( rest == opRes.c_str() )
 			{
 				throw std::runtime_error("malformed testcase: missing tolerated area difference in 'areatest' op");
@@ -1230,7 +1236,7 @@ XMLTester::parseTest(const TiXmlNode* node)
 
 			if ( testValidOutput )
 			{
-				testValid(gI.get(), "areatest intersection");
+				validOut &= testValid(gI.get(), "areatest intersection");
 			}
 
 			if ( verbose > 1 )
@@ -1245,6 +1251,11 @@ XMLTester::parseTest(const TiXmlNode* node)
 					overlayOp(OverlayOp::opDIFFERENCE));
 #endif
 
+			if ( testValidOutput )
+			{
+				validOut &= testValid(gI.get(), "areatest difference(a,b)");
+			}
+
 			if ( verbose > 1 )
 			{
 		std::cerr << "Running difference(B,A) for areatest" << std::endl;
@@ -1259,7 +1270,7 @@ XMLTester::parseTest(const TiXmlNode* node)
 
 			if ( testValidOutput )
 			{
-				testValid(gI.get(), "areatest difference");
+				validOut &= testValid(gI.get(), "areatest difference(b,a)");
 			}
 
 			if ( verbose > 1 )
@@ -1274,6 +1285,11 @@ XMLTester::parseTest(const TiXmlNode* node)
 					overlayOp(OverlayOp::opSYMDIFFERENCE));
 #endif
 
+			if ( testValidOutput )
+			{
+				validOut &= testValid(gI.get(), "areatest symdifference");
+			}
+
 			if ( verbose > 1 )
 			{
 		std::cerr << "Running union for areatest" << std::endl;
@@ -1332,7 +1348,7 @@ XMLTester::parseTest(const TiXmlNode* node)
 
 			if ( maxdiff <= toleratedDiff )
 			{
-				success=1;
+				success = 1 && validOut;
 			}
 
 			std::stringstream tmp;
@@ -1341,6 +1357,24 @@ XMLTester::parseTest(const TiXmlNode* node)
 			expected_result=opRes;
 
 		}
+		else if (opName=="distance")
+		{
+			char* rest;
+			double distE = std::strtod(opRes.c_str(), &rest);
+			if ( rest == opRes.c_str() )
+			{
+				throw std::runtime_error("malformed testcase: missing expected result in 'distance' op");
+			}
+
+			geom::Geometry *g1 = opArg1 == "B" ? gB : gA;
+			geom::Geometry *g2 = opArg2 == "B" ? gB : gA;
+			double distO = g1->distance(g2);
+			std::stringstream ss; ss << distO; 
+			actual_result = ss.str();
+
+			// TODO: Use a tolerance ?
+			success = ( distO == distE ) ? 1 : 0;
+		}
 
 		else
 		{
@@ -1362,7 +1396,9 @@ XMLTester::parseTest(const TiXmlNode* node)
 	}
 	catch (...)
 	{
-		std::cerr<<"EXEPTION"<<std::endl;
+		std::cerr << "Unknown EXEPTION on case "
+		          << caseCount
+		          << std::endl;
 		actual_result = "Unknown exception thrown";
 	}
 
@@ -1462,7 +1498,12 @@ main(int argC, char* argV[])
 		}
 
 		std::string source = argV[i];
-		tester.run(source);
+
+		try {
+			tester.run(source);
+		} catch (const std::exception& exc) {
+			std::cerr<<exc.what()<<std::endl;
+		}
 	}
 
 	if ( ! sql_output ) tester.resultSummary(std::cout);
